using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Telerik.WebControls;
using System.Data.SqlClient;
using System.Data.SqlTypes;
using System.IO;

public partial class RadUploadInAjaxifiedGrid_DefaultCS : System.Web.UI.Page
{
    private static string _connectionString = null;

    public static int MAX_THUMBNAIL_WIDTH = 60;
    public static int MAX_THUMBNAIL_HEIGHT = 60;

    #region Initialization

    private SqlConnection CreateConnection()
    {
        _connectionString = ConfigurationManager.ConnectionStrings["demoConnectionString1"].ToString();
        return new SqlConnection(_connectionString);
    }

    #endregion

    protected void Page_Load(object sender, EventArgs e)
    {
        if (!IsPostBack)
        {
            RadGrid1.MasterTableView.DataKeyNames = new string[] { "ID" };
        }
    }

    #region Files

    protected void SaveFileAndThumbnail(UploadedFile imageFile, int id)
    {
        imageFile.SaveAs(GetFileName(id, imageFile.GetExtension()), true);
        CreateThumbnail(imageFile.InputStream, GetThumbnailName(id, imageFile.GetExtension()));
    }

    private string GetFileName(int id, string extension)
    {
        return Path.Combine(Server.MapPath("Images"), String.Concat(id, extension));
    }

    private string GetThumbnailName(int id, string extension)
    {
        return Path.Combine(Server.MapPath("Thumbnails"), String.Concat(id, extension));
    }

    protected void DeleteFileAndThumbnail(int id)
    {
        foreach (string imageFile in Directory.GetFiles(Server.MapPath("Images"), String.Concat(id, ".*")))
        {
            File.Delete(imageFile);
        }
        foreach (string imageFile2 in Directory.GetFiles(Server.MapPath("Thumbnails"), String.Concat(id, ".*")))
        {
            File.Delete(imageFile2);
        }
    }

    #endregion

    #region DB Code

    private void InsertDb(UploadedFile imageFile, string description, string name, int authorId)
    {
        SqlConnection conn = CreateConnection();
        try
        {
            conn.Open();

            SqlCommand command = null;
            try
            {
                command = new SqlCommand("INSERT INTO Images ([Name], [Description], [Extension], [AuthorNameID]) VALUES (@Name, @Description, @Extension, @AuthorId) SELECT @@identity", conn);

                command.Parameters.AddWithValue("@Name", name);
                command.Parameters.AddWithValue("@Description", description);
                command.Parameters.AddWithValue("@Extension", imageFile.GetExtension());
                command.Parameters.AddWithValue("@AuthorId", authorId);
                int imageID = (int)(decimal)command.ExecuteScalar();

                SaveFileAndThumbnail(imageFile, imageID);
            }
            finally
            {
                if (command != null) command.Dispose();
            }
        }
        finally
        {
            if (conn.State == ConnectionState.Open)
            {
                conn.Close();
            }
        }
    }

    private void UpdateDb(int id, UploadedFile imageFile, string description, string newName, int authorId)
    {
        SqlConnection conn = CreateConnection();
        try
        {
            conn.Open();

            SqlCommand command = null;
            try
            {
                if (imageFile != null)
                {
                    command = new SqlCommand("UPDATE Images SET [Name]=@Name, [Description]=@Description, [Extension]=@Extension, [AuthorNameID]=@AuthorId WHERE [ID]= @ID", conn);
                    command.Parameters.AddWithValue("@Extension", imageFile.GetExtension());

                    DeleteFileAndThumbnail(id);
                    SaveFileAndThumbnail(imageFile, id);
                }
                else
                {
                    command = new SqlCommand("UPDATE Images SET [Name]=@Name, [Description]=@Description, [AuthorNameID]=@AuthorId WHERE [ID]= @ID", conn);
                }
                command.Parameters.AddWithValue("@ID", id);
                command.Parameters.AddWithValue("@Name", newName);
                command.Parameters.AddWithValue("@Description", description);
                command.Parameters.AddWithValue("@AuthorId", authorId);
                command.ExecuteNonQuery();
            }
            finally
            {
                if (command != null) command.Dispose();
            }
        }
        finally
        {
            if (conn.State == ConnectionState.Open)
            {
                conn.Close();
            }
        }
    }

    private void DeleteImageDb(int id)
    {
        SqlConnection conn = CreateConnection();
        try
        {
            conn.Open();

            SqlCommand command = new SqlCommand("DELETE FROM Images WHERE [ID]= @ID", conn);
            command.Parameters.AddWithValue("@ID", id);
            command.ExecuteNonQuery();
        }
        finally
        {
            if (conn.State == ConnectionState.Open)
            {
                conn.Close();
            }
        }
    }

    #endregion

    #region RadGrid Command Handlers

    protected void RadGrid1_UpdateCommand(object source, GridCommandEventArgs e)
    {
        if (IsValid)
        {
            RadUpload uploader = e.Item.FindControl("RadUpload1") as RadUpload;
            TextBox imageDescription = e.Item.FindControl("txbDescription") as TextBox;
            TextBox imageName = e.Item.FindControl("txbName") as TextBox;
            RadComboBox author = e.Item.FindControl("AuthorImagesList") as RadComboBox;

            int id = (int)RadGrid1.MasterTableView.DataKeyValues[e.Item.ItemIndex]["ID"];

            UploadedFile imageFile = null;
            if (uploader.UploadedFiles.Count > 0)
            {
                imageFile = uploader.UploadedFiles[0];
            }

            UpdateDb(id, imageFile, imageDescription.Text, imageName.Text, Int32.Parse(author.Value));
        }
    }

    protected void RadGrid1_InsertCommand(object source, GridCommandEventArgs e)
    {
        if (IsValid)
        {
            RadUpload uploader = e.Item.FindControl("RadUpload1") as RadUpload;
            TextBox imageDescription = e.Item.FindControl("txbDescription") as TextBox;
            TextBox imageName = e.Item.FindControl("txbName") as TextBox;
            RadComboBox author = e.Item.FindControl("AuthorImagesList") as RadComboBox;

            UploadedFile imageFile = null;

            if (uploader.UploadedFiles.Count > 0)
            {
                imageFile = uploader.UploadedFiles[0];
                InsertDb(imageFile, imageDescription.Text, imageName.Text, Int32.Parse(author.Value));
            }

        }
    }

    protected void RadGrid1_DeleteCommand(object source, GridCommandEventArgs e)
    {
        int id = (int)RadGrid1.MasterTableView.DataKeyValues[e.Item.ItemIndex]["ID"];
        DeleteImageDb(id);
        DeleteFileAndThumbnail(id);
    }

    protected void RadGrid1_ItemDataBound(object sender, GridItemEventArgs e)
    {
        if (e.Item is GridEditableItem && e.Item.IsInEditMode)
        {
            HiddenField currentAuthorName = e.Item.FindControl("initialAuthorName") as HiddenField;
            HiddenField currentAuthorValue = e.Item.FindControl("initialAuthorValue") as HiddenField;
            RadComboBox author = e.Item.FindControl("AuthorImagesList") as RadComboBox;
            RadUpload upload = e.Item.FindControl("RadUpload1") as RadUpload;
            author.Items[0].Text = currentAuthorName.Value;
            author.Items[0].Value = currentAuthorValue.Value;
            string labelText = string.Format("<script type='text/javascript'>window['AuthorListId'] = {0};window['UploadId'] = {1};</script>", author.ClientID, upload.ClientID);
            RadGrid1.Controls.Add(new LiteralControl(labelText));
        }
    }

    #endregion

    #region Thumbnails

    private void CreateThumbnail(Stream imageStream, string thumbnailName)
    {
        System.Drawing.Image originalImage = null;
        try
        {
            originalImage = System.Drawing.Image.FromStream(imageStream);
            int[] realThumbDimensions = GetProportionalThumbnailSize(originalImage.Width, originalImage.Height);

            System.Drawing.Image thumbnail = null;
            try
            {
                thumbnail = originalImage.GetThumbnailImage(
                    realThumbDimensions[0],
                    realThumbDimensions[1],
                    null,
                    IntPtr.Zero);
                thumbnail.Save(thumbnailName);
            }
            finally
            {
                if (thumbnail != null) thumbnail.Dispose();
            }
        }
        finally
        {
            if (originalImage != null) originalImage.Dispose();
        }
    }

    private int[] GetProportionalThumbnailSize(int originalWidth, int originalHeight)
    {
        int realWidth = MAX_THUMBNAIL_WIDTH;
        int realHeight = MAX_THUMBNAIL_HEIGHT;

        if (originalWidth > originalHeight)
        {
            realHeight = (int)Math.Round((decimal)(originalHeight * MAX_THUMBNAIL_WIDTH / originalWidth));
        }
        else
        {
            realWidth = (int)Math.Round((decimal)(originalWidth * MAX_THUMBNAIL_HEIGHT / originalHeight));
        }
        return new int[] { realWidth, realHeight };
    }

    #endregion

}
